@@ -92,6 +92,7 @@ func NewConfigObserver(
 			apiserver.ObserveAdditionalCORSAllowedOrigins,
 			libgoapiserver.ObserveTLSSecurityProfile,
 			auth.ObserveAuthMetadata,
+			auth.ObserveServiceAccountIssuer,
 			encryption.NewEncryptionConfigObserver(
 				operatorclient.TargetNamespace,
 				// static path at which we expect to find the encryption config secret
@@ -102,7 +103,11 @@ func NewConfigObserver(
 				"openshift-kube-apiserver",
 				[]string{"apiServerArguments", "cloud-provider"},
 				[]string{"apiServerArguments", "cloud-config"}),
-			featuregates.NewObserveFeatureFlagsFunc(nil, []string{"apiServerArguments", "feature-gates"}),
+			featuregates.NewObserveFeatureFlagsFunc(
+				nil,
+				nil,
+				[]string{"apiServerArguments", "feature-gates"},
+			),
 			network.ObserveRestrictedCIDRs,
 			network.ObserveServicesSubnet,
 			network.ObserveExternalIPPolicy,
