@@ -1,32 +1,18 @@
 package operator
 
 import (
+	"context"
 	"fmt"
 	"os"
 	"time"
 
-	"k8s.io/apimachinery/pkg/api/errors"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/client-go/dynamic"
-	"k8s.io/client-go/kubernetes"
-
+	kubemigratorclient "github.com/kubernetes-sigs/kube-storage-version-migrator/pkg/clients/clientset"
+	migrationv1alpha1informer "github.com/kubernetes-sigs/kube-storage-version-migrator/pkg/clients/informer"
 	configv1 "github.com/openshift/api/config/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
 	configv1client "github.com/openshift/client-go/config/clientset/versioned"
 	configv1informers "github.com/openshift/client-go/config/informers/externalversions"
-	"github.com/openshift/library-go/pkg/controller/controllercmd"
-	"github.com/openshift/library-go/pkg/operator/certrotation"
-	"github.com/openshift/library-go/pkg/operator/encryption"
-	"github.com/openshift/library-go/pkg/operator/encryption/controllers/migrators"
-	encryptiondeployer "github.com/openshift/library-go/pkg/operator/encryption/deployer"
-	"github.com/openshift/library-go/pkg/operator/genericoperatorclient"
-	"github.com/openshift/library-go/pkg/operator/staleconditions"
-	"github.com/openshift/library-go/pkg/operator/staticpod"
-	"github.com/openshift/library-go/pkg/operator/staticpod/controller/revision"
-	"github.com/openshift/library-go/pkg/operator/status"
-	"github.com/openshift/library-go/pkg/operator/v1helpers"
-
+	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/boundsatokensignercontroller"
 	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/certrotationcontroller"
 	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/certrotationtimeupgradeablecontroller"
 	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/configmetrics"
@@ -35,22 +21,34 @@ import (
 	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/operatorclient"
 	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/resourcesynccontroller"
 	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/targetconfigcontroller"
+	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/terminationobserver"
+	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/v410_00_assets"
+	"github.com/openshift/library-go/pkg/controller/controllercmd"
+	"github.com/openshift/library-go/pkg/operator/certrotation"
+	"github.com/openshift/library-go/pkg/operator/encryption"
+	"github.com/openshift/library-go/pkg/operator/encryption/controllers/migrators"
+	encryptiondeployer "github.com/openshift/library-go/pkg/operator/encryption/deployer"
+	"github.com/openshift/library-go/pkg/operator/genericoperatorclient"
+	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
+	"github.com/openshift/library-go/pkg/operator/staleconditions"
+	"github.com/openshift/library-go/pkg/operator/staticpod"
+	"github.com/openshift/library-go/pkg/operator/staticpod/controller/revision"
+	"github.com/openshift/library-go/pkg/operator/staticresourcecontroller"
+	"github.com/openshift/library-go/pkg/operator/status"
+	"github.com/openshift/library-go/pkg/operator/v1helpers"
+	"k8s.io/apimachinery/pkg/api/errors"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/client-go/kubernetes"
 )
 
-func RunOperator(ctx *controllercmd.ControllerContext) error {
+func RunOperator(ctx context.Context, controllerContext *controllercmd.ControllerContext) error {
 	// This kube client use protobuf, do not use it for CR
-	kubeClient, err := kubernetes.NewForConfig(ctx.ProtoKubeConfig)
+	kubeClient, err := kubernetes.NewForConfig(controllerContext.ProtoKubeConfig)
 	if err != nil {
 		return err
 	}
-	migrationClientConfig := dynamic.ConfigFor(ctx.KubeConfig)
-	migrationClientConfig.Burst = 40
-	migrationClientConfig.QPS = 30
-	dynamicClientForMigration, err := dynamic.NewForConfig(migrationClientConfig)
-	if err != nil {
-		return err
-	}
-	configClient, err := configv1client.NewForConfig(ctx.KubeConfig)
+	configClient, err := configv1client.NewForConfig(controllerContext.KubeConfig)
 	if err != nil {
 		return err
 	}
@@ -64,7 +62,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		"openshift-etcd",
 	)
 	configInformers := configv1informers.NewSharedInformerFactory(configClient, 10*time.Minute)
-	operatorClient, dynamicInformers, err := genericoperatorclient.NewStaticPodOperatorClient(ctx.KubeConfig, operatorv1.GroupVersion.WithResource("kubeapiservers"))
+	operatorClient, dynamicInformers, err := genericoperatorclient.NewStaticPodOperatorClient(controllerContext.KubeConfig, operatorv1.GroupVersion.WithResource("kubeapiservers"))
 	if err != nil {
 		return err
 	}
@@ -73,7 +71,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		operatorClient,
 		kubeInformersForNamespaces,
 		kubeClient,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 	if err != nil {
 		return err
@@ -84,9 +82,25 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		kubeInformersForNamespaces,
 		configInformers,
 		resourceSyncController,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 
+	staticResourceController := staticresourcecontroller.NewStaticResourceController(
+		"KubeAPIServerStaticResources",
+		v410_00_assets.Asset,
+		[]string{
+			"v4.1.0/kube-apiserver/ns.yaml",
+			"v4.1.0/kube-apiserver/svc.yaml",
+			"v4.1.0/kube-apiserver/kubeconfig-cm.yaml",
+			"v4.1.0/kube-apiserver/localhost-recovery-client-crb.yaml",
+			"v4.1.0/kube-apiserver/localhost-recovery-sa.yaml",
+			"v4.1.0/kube-apiserver/localhost-recovery-token.yaml",
+		},
+		(&resourceapply.ClientHolder{}).WithKubernetes(kubeClient),
+		operatorClient,
+		controllerContext.EventRecorder,
+	).AddKubeInformers(kubeInformersForNamespaces)
+
 	targetConfigReconciler := targetconfigcontroller.NewTargetConfigController(
 		os.Getenv("IMAGE"),
 		os.Getenv("OPERATOR_IMAGE"),
@@ -94,7 +108,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace),
 		kubeInformersForNamespaces,
 		kubeClient,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 
 	// don't change any versions until we sync
@@ -109,7 +123,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 	versionRecorder.SetVersion("raw-internal", status.VersionForOperatorFromEnv())
 
 	staticPodControllers, err := staticpod.NewBuilder(operatorClient, kubeClient, kubeInformersForNamespaces).
-		WithEvents(ctx.EventRecorder).
+		WithEvents(controllerContext.EventRecorder).
 		WithInstaller([]string{"cluster-kube-apiserver-operator", "installer"}).
 		WithPruning([]string{"cluster-kube-apiserver-operator", "prune"}, "kube-apiserver-pod").
 		WithResources(operatorclient.TargetNamespace, "kube-apiserver", RevisionConfigMaps, RevisionSecrets).
@@ -124,6 +138,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		"kube-apiserver",
 		[]configv1.ObjectReference{
 			{Group: "operator.openshift.io", Resource: "kubeapiservers", Name: "cluster"},
+			{Group: "apiextensions.k8s.io", Resource: "customresourcedefinitions"},
 			{Resource: "namespaces", Name: operatorclient.GlobalUserSpecifiedConfigNamespace},
 			{Resource: "namespaces", Name: operatorclient.GlobalMachineSpecifiedConfigNamespace},
 			{Resource: "namespaces", Name: operatorclient.OperatorNamespace},
@@ -133,7 +148,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		configInformers.Config().V1().ClusterOperators(),
 		operatorClient,
 		versionRecorder,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 
 	certRotationScale, err := certrotation.GetCertRotationScale(kubeClient, operatorclient.GlobalUserSpecifiedConfigNamespace)
@@ -146,7 +161,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		operatorClient,
 		configInformers,
 		kubeInformersForNamespaces,
-		ctx.EventRecorder.WithComponentSuffix("cert-rotation-controller"),
+		controllerContext.EventRecorder.WithComponentSuffix("cert-rotation-controller"),
 		certRotationScale,
 	)
 	if err != nil {
@@ -158,7 +173,10 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 	if err != nil {
 		return err
 	}
-	migrator := migrators.NewInProcessMigrator(dynamicClientForMigration, kubeClient.Discovery())
+
+	migrationClient := kubemigratorclient.NewForConfigOrDie(controllerContext.KubeConfig)
+	migrationInformer := migrationv1alpha1informer.NewSharedInformerFactory(migrationClient, time.Minute*30)
+	migrator := migrators.NewKubeStorageVersionMigrator(migrationClient, migrationInformer.Migration().V1alpha1(), kubeClient.Discovery())
 
 	encryptionControllers, err := encryption.NewControllers(
 		operatorclient.TargetNamespace,
@@ -169,7 +187,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		configInformers.Config().V1().APIServers(),
 		kubeInformersForNamespaces,
 		kubeClient.CoreV1(),
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 		schema.GroupResource{Group: "", Resource: "secrets"},
 		schema.GroupResource{Group: "", Resource: "configmaps"},
 	)
@@ -180,17 +198,35 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 	featureUpgradeableController := featureupgradablecontroller.NewFeatureUpgradeableController(
 		operatorClient,
 		configInformers,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 
 	certRotationTimeUpgradeableController := certrotationtimeupgradeablecontroller.NewCertRotationTimeUpgradeableController(
 		operatorClient,
 		kubeInformersForNamespaces.InformersFor(operatorclient.GlobalUserSpecifiedConfigNamespace).Core().V1().ConfigMaps(),
-		ctx.EventRecorder.WithComponentSuffix("cert-rotation-controller"),
+		controllerContext.EventRecorder.WithComponentSuffix("cert-rotation-controller"),
+	)
+
+	terminationObserver := terminationobserver.NewTerminationObserver(
+		operatorclient.TargetNamespace,
+		kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace),
+		kubeClient.CoreV1(),
+		controllerContext.EventRecorder,
+	)
+
+	boundSATokenSignerController := boundsatokensignercontroller.NewBoundSATokenSignerController(
+		operatorClient,
+		kubeInformersForNamespaces,
+		kubeClient,
+		controllerContext.EventRecorder,
 	)
 
 	staleConditionsController := staleconditions.NewRemoveStaleConditions(
 		[]string{
+			// the static pod operator used to directly set these. this removes those conditions since the static pod operator was updated.
+			// these can be removed in 4.5
+			"Available", "Progressing",
+
 			// We removed the controller that sets the condition `DefaultSecurityContextConstraintsUpgradeable` if any
 			// default SCC has been mutated.
 			// A cluster that already has user-modified default SCCs will have the above condition set. We need to
@@ -198,26 +234,33 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 			"DefaultSecurityContextConstraintsUpgradeable",
 		},
 		operatorClient,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 
+	// register termination metrics
+	terminationobserver.RegisterMetrics()
+
 	// register config metrics
 	configmetrics.Register(configInformers)
 
 	kubeInformersForNamespaces.Start(ctx.Done())
 	configInformers.Start(ctx.Done())
 	dynamicInformers.Start(ctx.Done())
+	migrationInformer.Start(ctx.Done())
 
-	go staticPodControllers.Run(ctx.Done())
-	go resourceSyncController.Run(1, ctx.Done())
+	go staticPodControllers.Run(ctx, 1)
+	go resourceSyncController.Run(ctx, 1)
+	go staticResourceController.Run(ctx, 1)
 	go targetConfigReconciler.Run(1, ctx.Done())
-	go configObserver.Run(1, ctx.Done())
-	go clusterOperatorStatus.Run(1, ctx.Done())
-	go certRotationController.Run(1, ctx.Done())
+	go configObserver.Run(ctx, 1)
+	go clusterOperatorStatus.Run(ctx, 1)
+	go certRotationController.Run(ctx, 1)
 	go encryptionControllers.Run(ctx.Done())
 	go featureUpgradeableController.Run(1, ctx.Done())
 	go certRotationTimeUpgradeableController.Run(1, ctx.Done())
-	go staleConditionsController.Run(1, ctx.Done())
+	go terminationObserver.Run(ctx, 1)
+	go boundSATokenSignerController.Run(ctx)
+	go staleConditionsController.Run(ctx, 1)
 
 	<-ctx.Done()
 	return fmt.Errorf("stopped")
@@ -233,6 +276,12 @@ var RevisionConfigMaps = []revision.RevisionResource{
 	{Name: "oauth-metadata", Optional: true},
 	{Name: "cloud-config", Optional: true},
 
+	// This configmap is managed by the operator, but ensuring a revision history
+	// supports signing key promotion. Promotion requires knowing whether the current
+	// public key is present in the configmap(s) associated with the current
+	// revision(s) of the master nodes.
+	{Name: "bound-sa-token-signing-certs"},
+
 	// these need to removed, but if we remove them now, the cluster will die because we don't reload them yet
 	{Name: "etcd-serving-ca"},
 	{Name: "kube-apiserver-server-ca", Optional: true},
@@ -244,11 +293,13 @@ var RevisionConfigMaps = []revision.RevisionResource{
 var RevisionSecrets = []revision.RevisionResource{
 	// these need to removed, but if we remove them now, the cluster will die because we don't reload them yet
 	{Name: "etcd-client"},
-	// this is needed so that the cert syncer itself can request certs.  It uses localhost
-	{Name: "kube-apiserver-cert-syncer-client-cert-key"},
 	{Name: "kubelet-client"},
 	// etcd encryption
 	{Name: "encryption-config", Optional: true},
+
+	// this needs to be revisioned as certsyncer's kubeconfig isn't wired to be live reloaded, nor will be autorecovery
+	{Name: "localhost-recovery-serving-certkey"},
+	{Name: "localhost-recovery-client-token"},
 }
 
 var CertConfigMaps = []revision.RevisionResource{
@@ -265,7 +316,7 @@ var CertSecrets = []revision.RevisionResource{
 	{Name: "service-network-serving-certkey"},
 	{Name: "external-loadbalancer-serving-certkey"},
 	{Name: "internal-loadbalancer-serving-certkey"},
-	{Name: "localhost-recovery-serving-certkey"},
+	{Name: "bound-service-account-signing-key"},
 
 	{Name: "user-serving-cert", Optional: true},
 	{Name: "user-serving-cert-000", Optional: true},
